﻿namespace U2UConsult.WinRT.MVVMSlider.ViewModels
{
    using U2UConsult.WinRT.MVVMSlider.Common;

    public class MainPageViewModel : BindableBase
    {
        private bool isCelsius;
        private double someTemperature;
        private double anotherTemperature;

        public bool IsCelsius
        {
            get { return this.isCelsius; }
            set
            {
                if (this.isCelsius == value)
                {
                    return;
                }

                this.SetProperty(ref this.isCelsius, value);
                this.ChangeTemperatures();
            }
        }

        public string Unit
        {
            get
            {
                return this.isCelsius ? "°C" : "°F";
            }
        }

        public double MinimumTemperature
        {
            get
            {
                return this.isCelsius ? 0 : 32;
            }
        }

        public double MaximumTemperature
        {
            get
            {
                return this.isCelsius ? 28 : 82;
            }
        }

        public double SomeTemperature
        {
            get { return someTemperature; }
            set { this.SetProperty(ref this.someTemperature, value); }
        }

        public double AnotherTemperature
        {
            get { return anotherTemperature; }
            set { this.SetProperty(ref this.anotherTemperature, value); }
        }

        private void ChangeTemperatures()
        {        
            var temp = this.anotherTemperature;
            
            //
            // Harmless: not bound to the slider
            //
            
            this.OnPropertyChanged("Unit");

            //
            // Stupid: slider breaks when Mimimum gets greater than Maximum, so you have to do it in the correct order.
            //

            // Try this first, observe how it blocks the sliders while (re-)converting to Fahrenheit. 
            // this.OnPropertyChanged("MaximumTemperature");
            // this.OnPropertyChanged("MinimumTemperature");

            // Or this one, observe how it screws up the value of the top slider, *and* blocks the sliders while (re-)converting to Fahrenheit. 
            // this.OnPropertyChanged("MinimumTemperature");
            // this.OnPropertyChanged("MaximumTemperature");

            // Workaround: control change propagation order.
            if (this.isCelsius)
            {
                this.OnPropertyChanged("MinimumTemperature");
                this.OnPropertyChanged("MaximumTemperature");
            }
            else
            {
                this.OnPropertyChanged("MaximumTemperature");
                this.OnPropertyChanged("MinimumTemperature");
            }

            //
            // Deadly: slider updates the value when it gets out of range.
            //
            
            // This would be standard code. Unfortunately it does not work.
            if (this.isCelsius)
            {
                this.SomeTemperature = UnitConverter.FahrenheitToCelsius(this.SomeTemperature);
            }
            else
            {
                this.SomeTemperature = UnitConverter.CelsiusToFahrenheit(this.SomeTemperature);
            }

            // Workaround: backup value.
            if (this.isCelsius)
            {
                this.AnotherTemperature = UnitConverter.FahrenheitToCelsius(temp);
            }
            else
            {
                this.AnotherTemperature = UnitConverter.CelsiusToFahrenheit(temp);
            }
        }
    }
}
